#! /usr/bin/env python
# -*- coding: UTF-8 -*-

from edt2ics.ical import iCalSchedule
from edt2ics.scraper import ScheduleScraper

__version__ = '0.1.1'

# silent pyflakes
iCalSchedule, ScheduleScraper
